import traceback
from hombre.utils.postgres_wrapper import execute_query

tables_to_delete = [
    "policynsxconfigepoch",
    "normalizedcomputeconfig",
    "normalizedgroupconfig",
    "dfwrulecomputerelationshipconfig",
    "groupcomputerelationshipconfig",
    "groupipsetrelationshipconfig",
    "idssignature",
    "normalizeddfwpolicyconfig",
    "normalizeddfwruleconfig",
    "normalizeddomainconfig",
    "normalizedsiteconfig",
    "privateipranges",
    "l7info",
    "ipsetmemberinfo"
]


def execute():
    # Check if site_id exists as column in policynsxconfigepoch table,
    # if yes then the table schema has been migrated already,
    # else drop the required tables so that these tables would be created with new schema containing site_id.
    query = """
    SELECT column_name FROM information_schema.columns
    WHERE table_name='policynsxconfigepoch' and column_name='site_id';
    """
    try:
        result = execute_query(query_string=query, params="", fetch_post_exec=True)
    except Exception as e:
        print("Got exception while executing request: " + str(e))
        print(traceback.format_exc())
        raise

    if len(result) == 0:
        print("site_id is not present in epoch table, deleting config data from postgres")
        delete_config_data()
    else:
        print("site_id is already present in epoch table, skipping deletion of tables")


def delete_config_data():
    print("Dropping tables in postgres which do not contain siteId.")
    for table in tables_to_delete:
        query = "drop table if exists {} ;".format(table)
        try:
            execute_query(query_string=query, params="")
            print("Successfully dropped table : " + table)
        except Exception as e:
            print("Got exception while executing request: " + str(e))
            print(traceback.format_exc())
            raise
