import os
from datetime import datetime, timezone
from hombre.constants import CLUSTER_API_HOST
from hombre.constants import NSX_MANAGER_HOST
from hombre.constants import Url
from hombre.utils import requests_wrapper
from hombre.utils.postgres_wrapper import execute_query

NODE_CERT_RETRY_COUNT = 20


def ui_plugin(unregister=False):
    try:
        # create request parameters
        cluster_api_port = os.environ["CLUSTER_API_SERVICE_PORT"]
        url = Url.InCluster.POST_REGISTER_UI_PLUGIN.format(
            service_host=CLUSTER_API_HOST,
            service_port=cluster_api_port)
        req_data = {
            "service_names": ["intelligence"],
            "action": "unregister" if unregister else "register",
        }
        print("Calling intelligence UI plugin (un)registration API on cluster-api service")
        requests_wrapper.incluster_request('POST', url, data=req_data)
    except Exception as e:
        print(e)
        raise


def cluster(unregister=False):
    try:
        # get cluster-id from NSX
        url = Url.Egress.GET_CLUSTER_REGISTRATION
        print("Calling GET cluster registration.")
        json_response = requests_wrapper.egress_request("GET", url)
        cluster_id = None
        for cluster_item in json_response["napp_registration_results"]:
            cluster_id = cluster_item["cluster_id"]
        if not cluster_id:
            raise Exception("NSX Application Platform cluster registration not found")

        # PATCH is_intelligence_enabled for the given cluster_id
        url = Url.Egress.PATCH_CLUSTER_REGISTRATION.format(
            service_host=NSX_MANAGER_HOST,
            cluster_id=cluster_id)
        req_data = {
            "cluster_id": cluster_id,
            "is_intelligence_enabled": False if unregister else True
        }
        requests_wrapper.egress_request("PATCH", url, data=req_data, headers={'X-Allow-Overwrite': 'true'})
    except Exception as e:
        print(e)
        raise


def site():
    site_id = _get_site_id()
    _add_site(site_id)


def site_kindtest(site_id: str):
    _add_site(site_id)


def _get_site_id() -> str:
    try:
        url = Url.Egress.GET_SITE_ID
        json_response = requests_wrapper.egress_request("GET", url)
        print("GET site-id response is: {response}".format(response=json_response))
        site_id = json_response["site"]["id"]
        return site_id
    except Exception as e:
        print("Could not retrieve site_id from NSX Manager")
        print(e)
        raise


def _add_site(site_id: str):
    # NOTE(tandonp): site is always onboarded on-prem, so we hardcode
    # lm_status
    curr_time = datetime.now(timezone.utc)
    insertSiteIdQuery = """
    INSERT into dgs_sites (site_id, site_display_name, site_type, created_at, lm_status)
    VALUES (%s, %s, 'SITE', %s, 'ONBOARDED')
    ON CONFLICT ON CONSTRAINT dgs_sites_pkey DO NOTHING;
    """

    setOriginalSiteIdQuery = """
    ALTER TABLE public.dgs_sites ALTER COLUMN site_id SET DEFAULT %s;
    """

    try:
        execute_query(insertSiteIdQuery, (site_id, site_id, curr_time,))
        execute_query(setOriginalSiteIdQuery, (site_id,))
    except Exception as e:
        print(e)
        raise
