import os
import psycopg2


def execute_query(query_string, params, fetch_post_exec=False):
    """

    :param query_string: SQL query string
    :param params: params for the SQL query
    :param fetch_post_exec: boolean flag to query results post query execution.
                            causes error if query does not generate result.
    :return: (query result) if fetch_post_exec == True, else None
    """
    conn = _connect_to_db()
    try:
        with conn as c:
            with c.cursor() as curs:
                curs.execute(query_string, params)
                return curs.fetchall() if fetch_post_exec else None
    except Exception:
        raise
    finally:
        conn.close()


def _connect_to_db():
    config = PostgresConfig()
    if config.ssl:
        sslmode = "require"
    else:
        sslmode = "disable"
    conn = psycopg2.connect(dbname=config.db, user=config.user, password=config.password,
                            host=config.host, port=config.port, sslmode=sslmode)
    return conn


class PostgresConfig:
    def __init__(self, ssl_enabled=True):
        self.db = os.environ["POSTGRES_DB"]
        self.user = os.environ["POSTGRES_USERNAME"]
        self.password = os.environ["POSTGRES_PASSWORD"]
        self.port = os.environ["POSTGRES_PORT"]
        self.host = os.environ["POSTGRES_HOST"]
        self.ssl = ssl_enabled
